#ifndef GRIDPERIODIC_H
#define GRIDPERIODIC_H
#include <iostream>
#include <vector>
#include "BoxIterator.h"
#include "Vect3d.h"
#include <math.h>
#include <assert.h>
#define PHANTOM_CHAIN //With no potentials, defining this saves time
#define STIFF_SIMULATION 
#define MUSCLE_SIMULATION 
#ifdef MUSCLE_SIMULATION
#define STIFF_SIMULATION 
#endif

using namespace std;

class GridPeriodic 
{
   protected:
      vector<int>* box;
      vect_i grid_num;
      int num_boxes;
      double lattice_const;
      double radius;//maximum interaction radius
      vect_f box_width;
      vect_f inv_box_width;
      int which_mon(vector<int> * bp,int ind) const;
      vector<int> * monomer_list;
 
   //private:
   public:
      long rem_count;
      int Stop;
      void clip(vect_f& r) const;
      int grid_index(vect_i b) const;
      int grid_index_not_periodic(vect_i b) const;
      GridPeriodic(vect_f Box_Width, double Lattice_Const);
      void get_box(vect_f r, vect_i& b) const;
      void remainder3d(vect_f in, vect_f &out) const;
      void set_max_radius(double radius);
      double get_max_radius() const;
      void erase(vect_i b, int ind) const;
      void put(vect_i b, int ind) const;
      vector<int> * get_neighbors(vect_f r) const;
      vect_i  getGridNum() const;
      vect_f getBoxWidth() const;
      bool inBox(vect_f& r) const;
      void printAll() ;
      void iterateWholeGrid(void (*func)(vector<int> * BoxPointer, vect_i * IntBox, int * flag));
      void isGridEmpty();
      long getRem_Count();
      void freeGrid();

};

inline int mod(int x, int l) {
   if (x >= 0) {
      return x%l;
   }
   else {
      return (x+1)%l + l-1;
   }
}

inline void GridPeriodic::get_box(vect_f r, vect_i& b) const {
    b.setX((int) floor((r.xcomp()/lattice_const)));
    b.setY((int)floor( (r.ycomp()/lattice_const)));
    b.setZ((int)floor( (r.zcomp()/lattice_const)));
}


inline int GridPeriodic::grid_index(vect_i b) const {
     int g_ind =  mod(b.xcomp(),grid_num.xcomp()) + 
                  grid_num.xcomp()*
                  (mod(b.ycomp(),grid_num.ycomp()) +grid_num.ycomp()*mod(b.zcomp(),grid_num.zcomp()));
     assert(g_ind < num_boxes);
   return  g_ind;
}

inline int GridPeriodic::grid_index_not_periodic(vect_i b) const {
   return b.xcomp() + grid_num.xcomp()*(b.ycomp() +grid_num.ycomp()*(b.zcomp()));
}

inline void GridPeriodic::clip(vect_f& r) const {}

inline double remainder(double x, double div, double inv_div) {
   double whole = x*inv_div;
   double rem = whole - floor(whole);
   if (rem > 0.5) rem = rem-1.0;
   return rem*div;
}

inline bool inside(double x,double low,double high) {
   if (x < low || x > high)
      return false;
      return true;
}


inline void GridPeriodic::remainder3d(vect_f in, vect_f &out) const {
   //rem_count++;
   out.set(
	 remainder(in.xcomp(),box_width.xcomp(),inv_box_width.xcomp()),
	 remainder(in.ycomp(),box_width.ycomp(),inv_box_width.ycomp()),
	 remainder(in.zcomp(),box_width.zcomp(),inv_box_width.zcomp())
	 );
}

   inline vect_f GridPeriodic::getBoxWidth() const{
      return box_width;
   }

   inline double GridPeriodic::get_max_radius() const{
      return radius;
   }

inline void GridPeriodic::set_max_radius(double radius){
   this->radius = radius;
}



inline int GridPeriodic::which_mon(vector<int> * bp,int ind) const {
  for(int i=0; i < bp->size();i++) {if ((*bp)[i] == ind) return i;}
  cout << "bead number" << ind <<  "couldn't be found in the specified box\n" << endl;
  return -1;
}


inline void GridPeriodic::erase(vect_i b, int ind) const {
   int indx = grid_index(b);
   vector<int> * bp = box + indx; 
   int nb = bp->size();
   int i = which_mon(bp, ind);
   if (i < (nb - 1)) {
	(*bp)[i] = (*bp)[nb-1];
    }
   bp->pop_back();
}

inline void GridPeriodic::put(vect_i b, int ind) const {
   int indx = grid_index(b);
   vector<int> * bp = box + indx; 
   bp->push_back(ind);
}

inline bool GridPeriodic::inBox(vect_f& r) const {
   double x = r.xcomp();
   double y = r.ycomp();
   double z = r.zcomp();
   if (x < 0 || x >= box_width.xcomp())
      return false;
   if (y < 0 || y >= box_width.ycomp())
      return false;
   if (z < 0 || z >= box_width.zcomp())
      return false;
   return true;
}

inline vector<int> * GridPeriodic::get_neighbors(vect_f r) const {
   vect_i middle_box;
   get_box(r,middle_box);
   vect_f r_max = r + radius*(x_hat + y_hat+z_hat);
   vect_f r_min = r - radius*(x_hat + y_hat+z_hat);
   vect_i i_min, i_max;
   get_box(r_max, i_max);
   get_box(r_min, i_min);
   //note the limits of iteration are included <= not <


   //vector<int> * monomer_list = new vector<int>();
   monomer_list->clear();

   int x_max = i_max.xcomp();
   int y_max = i_max.ycomp();
   int z_max = i_max.zcomp();

   int x_min = i_min.xcomp();
   int y_min = i_min.ycomp();
   int z_min = i_min.zcomp();


   //For c.o.m. conserving dynamics, split this into  two parts. One that takes
   //the mod and one that doesn't have to. Seems to speed it up a lot.
   if (r_max.xcomp() > box_width.xcomp() || r_max.ycomp() > box_width.ycomp() || r_max.zcomp() > box_width.zcomp()
	 || r_min.xcomp() < 0 || r_min.ycomp() < 0 || r_min.zcomp() < 0) {
      for(int x=x_min;x <= x_max; x++) {
	 for(int y=y_min;y <= y_max; y++) {
	    for(int z=z_min;z <= z_max; z++) {
	       vect_i  i_search(x,y,z);
	       int box_indx;
	       box_indx= grid_index(i_search);

	       vector<int> * bp = box + box_indx; 
	       for(int i=0; i < bp->size();i++)
	       {
		  int ind = (*bp)[i];
		  monomer_list->push_back(ind);
	       }

	    }
	 }
      } 
   } else {
      for(int x=x_min;x <= x_max; x++) {
	 for(int y=y_min;y <= y_max; y++) {
	    for(int z=z_min;z <= z_max; z++) {
	       vect_i  i_search(x,y,z);
	       int box_indx;
	       box_indx= grid_index_not_periodic(i_search);

	       vector<int> * bp = box + box_indx; 
	       for(int i=0; i < bp->size();i++)
	       {
		  int ind = (*bp)[i];
		  monomer_list->push_back(ind);
	       }

	    }
	 }
      } 
   }


   return monomer_list;
}

inline vect_i  GridPeriodic::getGridNum() const{
   return grid_num;
}



inline long GridPeriodic::getRem_Count() {
   return rem_count;
}



#endif// GRIDPERIODIC_H
