//#ifndef POLYMER_H
//#define POLYMER_H

#include <iostream>
#include <vector>
//#define double double
//#include "vect_f.h"
//#include "Vector3d.h"
#include "RandNumGenerator.h"
#include "GridPeriodic.h"
#include "Tension.h"
#include "TensionDamped.h"
#include "MyosinBinding.h"
//#include "Polymers.h"


class Polymer{
   friend class Polymers;
   protected:
      vect_f * Coords;
      vect_f * Velocities;
      vect_f * ClampedPositions;
      RandNumGenerator * Rand;
      GridPeriodic * grid;
      Polymers * polymers;
      int NumMon;
      int Head;
      int min_head;
      int max_tail;
      int * mark;
      double Repulsion;
      double SpringConstant;
      double * SpringConstantArray;
      double NNConstant;
      double ExtForce;
      double StepLength;
      long seed;
      double Beta;
      double MoveSize;
      double AcceptRate;
      double * QuantitiesToAverage;
      bool clamped;//Are we putting on a clamping potential
      double ClampStrength;
      double Height;
      double SoftCoreRadius;
      double HardCoreRadius;
      double SoftCoreRadiusSq;
      double HardCoreRadiusSq;
      double OneOverBlah;//1.0/(SoftCoreRadiusSq-HardCoreRadiusSq)
      double distsq(int i, int j); 
      double VClamp(vect_f & R,int Monomer);
      double EPointAndChainHard(vect_f & point, int monomer);
      inline double ePointAndChains(const vect_f & point, int monomer) const;
      double ePointAndChain_slow(const vect_f & point, int monomer) const;
      double bESpringNeighbors(vect_f & point, int monomer);
      double ESpring();
      double ERepulsive();
      double ActinSpacing;
      double V_actin;
      vect_f actin_dir;
      MyosinBinding * Bind1;
      MyosinBinding * Bind2;


   public:
      Polymer();
      Polymer(int NumMon, double StepLength, double Repulsion, long seed);
      Tension * tension;
      void setPolymers(Polymers * polymers);
      void setParams(int NumMon, double StepLength, double Repulsion);
      void set(int NumMon, double StepLength, double Repulsion);
      void set(int NumMon, double StepLength, double Repulsion, vect_f * Coords);
      void setCoords(vect_f * Coords);
      void setVelocities(vect_f * Velocities);
      void setClamp(bool clamped);
      void setClampStrength(double ClampStrength);
      void setClampedPositions(vect_f * ClampedPositions);
      void setBeta(double Beta) ;
      void setMoveSize(double MoveSize);
      void setCores(double SoftCoreRadius, double HardCoreRadius);
      void setSpringConstant(double SpringConstant);
      void setSpringConstantArray(double * SpringConstantArray);
      void setNNConstant(double SpringConstant);
      void setExtForce(double ExtForce);
      void setHeight(double Height);
      double getHeight();
      int getNumMon(); 
      vect_f * array_copy(vect_f * C, int num);
      vect_f * getCoords();
      void initRand();
      void initRand(vect_f init_pos, vect_f BoxSize);
      void initRand(vect_f init_pos);
      void initRand(vect_f init_pos, vect_f BoxSize, vect_f init_dir);
      vect_f StiffForces(int monomer) const;
      void actinForce(vect_f &r0, vect_f &r1, vect_f * f, bool fixed, double t) const ;
      void initStraight();
      void initStraight(vect_f init_pos);
      void initScrunched();
      void initScrunched(vect_f init_pos);
      void init(vect_f * InitPositions);
      vect_f * CenterOfMass();
      double RgSq();
      double REndEndSq();
      int fixedMove(int num_steps);
      void rotate3Mons(int middle, vect_f & new_pos);
      int jiggle(int num_steps);
      int jiggle_once();
      int reptate(int num_steps);
      void getXCoords(double * X);
      void getYCoords(double * Y);
      void getShiftedCoords(double * X, double * Y);
      void remove(int mon); 
      void insert(int mon); 
      int reptate_once();
      vect_f fPointAndChains(const vect_f & point, int monomer) const;
      vect_f getActinDir();
      vect_f * getForce(double t, vect_f * noise, vect_f &f_end);
      void setActinSpacing(double ActinSpacing);
      double getActinSpacing();
      MyosinBinding * getBind1();
      MyosinBinding * getBind2();

      //variant where move size is just the gaussian step length
      int reptateGaussian(int num_steps); 

      //For interface ParameterFindable
      void initQuantities(int numParams);

      double * getAveQuantities();

      //tunes to optimize step size
      double move(int NumSteps);

      double getAcceptanceRate();
      int reshuffle();
      vect_f nnforces(int monomer) const;
      vect_f extForces() const;
      vect_f beadForce(vect_f &r0, vect_f &r1) const;
      void putBinders(MyosinBinding * Bind1, MyosinBinding * Bind2);
      void  isMyosinBound(bool *);
      void setActinParams(double VActinDepth, vect_f & actin_dir, double ActinSpacing);
      void initPutChain(double ** InitR );
};

inline void Polymer::insert(int mon){
   polymers->addMonomer(this, mon, Coords[mon]);
}

inline void Polymer::remove(int mon) {
   polymers->removeMonomer(this, mon, Coords[mon]);
}


#if 1//brute force
inline double Polymer::ePointAndChains(const vect_f & point, int monomer) const {
   double e=0;
   int NumChains = polymers->getNumChains();
   int num_list = NumChains*NumMon;
   for(int i=0; i < num_list; i++) {
      int indx = i;
      int mon = polymers->getMonomer(indx);
      Polymer * pol = polymers->polymers + polymers->getPolymer(indx);
      if (mon == monomer && this == pol)
	 continue;
      vect_f pos = pol->Coords[mon];
      double dsq = pos.distsq(point);
      if (pos.distsq(point) < HardCoreRadiusSq)
	 return 1e10;
      if (dsq < SoftCoreRadiusSq)
	 e += Repulsion;
   }
   return e;
}
#else//using the grid
double Polymer::ePointAndChains(const vect_f & point, int monomer) const {
   double e=0;
   vector<int> * monomer_list = grid->get_neighbors(point);
   int num_list = monomer_list->size();
   for(int i=0; i < num_list; i++) {
      int indx = (*monomer_list)[i];
      int mon = polymers->getMonomer(indx);
      Polymer * pol = polymers->polymers + polymers->getPolymer(indx);
      if (mon == monomer && this == pol)
	 continue;
      vect_f pos = pol->Coords[mon];
      vect_f delta;
      grid->remainder3d(pos-point,delta);
      double dsq = delta*delta;
      if (dsq < HardCoreRadiusSq)
      {
	 //delete monomer_list;
	 return 1e10;
      }
      if (dsq < SoftCoreRadiusSq)
	 e += Repulsion;
   }
//   delete monomer_list;
   return e;
}
#endif

inline vect_f Polymer::nnforces(int monomer) const {
   if (monomer == 0)
      return NNConstant*(Coords[1]-Coords[0]);
   else if (monomer == (NumMon-1))
      return NNConstant*(Coords[NumMon-2]-Coords[NumMon-1]);
   else 
      return NNConstant*(-2.0*Coords[monomer]+Coords[monomer-1]+Coords[monomer+1]);
}

inline vect_f Polymer::extForces() const {
   vect_f r = Coords[NumMon-1] - Coords[0];
   double dist = sqrt(r*r);
   return (ExtForce*r)/(CUBE(dist));
}

#if 0//brute force
inline vect_f Polymer::fPointAndChains(const vect_f & point, int monomer) const {
   vect_f f=0;
   int NumChains = polymers->getNumChains();
   int num_list = NumChains*NumMon;
   for(int i=0; i < num_list; i++) {
      int indx = i;
      int mon = polymers->getMonomer(indx);
      Polymer * pol = polymers->polymers + polymers->getPolymer(indx);
      if (abs(mon - monomer) <= 1 && this == pol)
	 continue;

      vect_f pos = pol->Coords[mon];
      vect_f delta;
      grid->remainder3d(pos-point,delta);
      //double dsq = pos.distsq(point);
      double dsq = delta*delta;

      if (dsq < SoftCoreRadiusSq) {
	 f += -Repulsion * delta*QUART(SoftCoreRadiusSq-dsq);
	 //cout << "monomer  = " << monomer << " mon = " << mon << endl;
      }
   }
   return f;
}
#else//using the grid

inline vect_f Polymer::fPointAndChains(const vect_f & point, int monomer) const {
   vect_f f=0;
   if (monomer < 1)
      return f;
   vector<int> * monomer_list = grid->get_neighbors(point);
   int num_list = monomer_list->size();
   for(int i=0; i < num_list; i++) {
      int indx = (*monomer_list)[i];
      int mon = polymers->getMonomer(indx);
      Polymer * pol = polymers->polymers + polymers->getPolymer(indx);
      if (abs(mon - monomer) <= 1 && this == pol)
	 continue;
      vect_f pos = pol->Coords[mon];
      vect_f delta;
      //grid->remainder3d(pos-point,delta);
      //double dsq = delta*delta;
      double dsq = pos.distsq(point);
      if (dsq < SoftCoreRadiusSq) {
	 f += -Repulsion * delta*QUART(SoftCoreRadiusSq-dsq);
	 //cout << "monomer  = " << monomer << " mon = " << mon << endl;
      }
   }
//   delete monomer_list;
   return f;
}
#endif//use grid


   inline double Polymer::bESpringNeighbors(vect_f & point, int monomer) {
      if (monomer == 0)
	 return point.distsq(Coords[1]);
      else if (monomer == NumMon-1) 
	 return point.distsq(Coords[NumMon-2]);
      else
	 return point.distsq(Coords[monomer-1]) + point.distsq(Coords[monomer+1]);
   }

#if 0
inline vect_f Polymer::StiffForces(int monomer) const {
   if (monomer == 0)
      return SpringConstant*(
	    Coords[monomer+2]-2.*Coords[monomer+1]+Coords[monomer]);
   else if (monomer == 1)
	    SpringConstant*(Coords[monomer+2]-4.*Coords[monomer+1]+5.*Coords[monomer]-2.*Coords[monomer-1]);

   else if (monomer == (NumMon-1))
      return SpringConstant*(
	    Coords[monomer]-2.*Coords[monomer-1]+Coords[monomer-2]);
   else if (monomer == (NumMon-2))
      return SpringConstant*(
	    -2.*Coords[monomer+1]+5.*Coords[monomer]-4.*Coords[monomer-1]+Coords[monomer-2]);
   else 
      return 
	 SpringConstant*(Coords[monomer+2]-4.*Coords[monomer+1]+6.*Coords[monomer]-4.*Coords[monomer-1]+Coords[monomer-2]);
}
#else
#if 0
inline vect_f Polymer::StiffForces(int monomer) const {
   if (monomer == 0)
      return SpringConstant*(Coords[2]-Coords[0]);
   else if (monomer == 1)
      return SpringConstant*(Coords[3]-Coords[1]);
   else if (monomer == (NumMon-1))
      return SpringConstant*(Coords[NumMon-3]-Coords[NumMon-1]);
   else if (monomer == (NumMon-2))
      return SpringConstant*(Coords[NumMon-4]-Coords[NumMon-2]);
   else 
      return SpringConstant*(-2.0*Coords[monomer]+Coords[monomer-2]+Coords[monomer+2]);

}
#endif

inline vect_f Polymer::StiffForces(int monomer) const {
   vect_f f_r, f_l;
   if (monomer < 2)
      return  SpringConstantArray[monomer+1] * (Coords[monomer+2] - Coords[monomer]);
   if (monomer > NumMon-3)
      return SpringConstantArray[monomer-1] * (Coords[monomer -2] - Coords[monomer]);
   else
      return SpringConstantArray[monomer-1] * (Coords[monomer -2] - Coords[monomer])
            +    SpringConstantArray[monomer+1] * (Coords[monomer+2] - Coords[monomer]);
}
#endif

static double rem(double p, double q) {
   double frac = 0.5+p/q;
   return q*(frac - floor(frac) - 0.5);
}

//The first and last beads r0, r1:
vect_f inline Polymer::beadForce(vect_f &r0, vect_f &r1) const {

   vect_f diff = r0-r1;
   double dsq = diff*diff;
   if (dsq > SoftCoreRadiusSq) {
      vect_f ret = 0;
      return ret;
   }
   vect_f f_rep = 1000.0*diff*QUART(1.0/(dsq-HardCoreRadiusSq - OneOverBlah));
   return f_rep;

}

void inline Polymer::actinForce(vect_f &r0, vect_f &r1, vect_f * f, bool fixed, double t) const {
   if (fabs(r0.ycomp()) > SoftCoreRadius || fabs(r0.zcomp()) > SoftCoreRadius) {
      f[0] = f[1] = 0;
      return;
   }
   double rem_r;
   double ActVel = polymers->actin_velocity;
   if (fixed)
   {
      rem_r = r0.xcomp();
   }
   else
   {
      rem_r = rem(r0.xcomp()-ActVel*t, ActinSpacing);
   }
   vect_f dr;
   double invStepLength = 1.0/StepLength;
   dr.set(rem_r,r0.ycomp()-Height,r0.zcomp());
   double distsq = dr*dr;
   if (distsq < SoftCoreRadius) {
      double V_0  = -0.125*QUART(SoftCoreRadiusSq-distsq);
      vect_f f_0 = -2.0*dr*CUBE(SoftCoreRadiusSq-distsq);
      vect_f delta = (r1-r0)*invStepLength-actin_dir;
      double V_dir= 0.5*(delta*delta + 1);
      f[0] = V_actin*(V_0 * delta*invStepLength + V_dir * f_0);
      f[1] = -V_actin*(V_0 * delta*invStepLength);
      return;
   }
   else {
      f[0] = f[1] = 0;
      return;
   }
}

//#endif//POLYMER_H
