#include "graphics_on.h"
#ifdef GRAPHICS_ON
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "graphics.h"

//extern "C" {

static GRAPHICS  g;
/*
GLfloat mat_ambient_red[] = { 0.8, 0.0, 0.0, 1.0 };
GLfloat mat_ambient_green[] = { 0.0, 0.8, 0.0, 1.0 };
GLfloat mat_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
*/



void errorCallback(GLenum errorCode)
{
   const GLubyte *estring;

   estring = gluErrorString(errorCode);
   fprintf(stderr, "Quadric Error: %s\n", estring);
   exit(0);
}


//void init(GRAPHICS * graphics) 
void init() 
{
   GLUquadricObj *qobj;
   GLfloat mat_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
   GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat mat_shininess[] = { 50.0 };
   GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
   GLfloat model_ambient[] = { 0.5, 0.5, 0.5, 1.0 };

   glClearColor(0.0, 0.0, 0.0, 0.0);

   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
   glLightfv(GL_LIGHT0, GL_POSITION, light_position);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, model_ambient);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_DEPTH_TEST);
   qobj = gluNewQuadric();
   g.qobj  = qobj;
}


void graph_sphere(double * center, double radius)
{

   glPushMatrix();
   gluQuadricCallback(g.qobj, GLU_ERROR, (void (*) ()) errorCallback);
   gluQuadricDrawStyle(g.qobj, GLU_FILL); /* smooth shaded */
   gluQuadricNormals(g.qobj, GLU_SMOOTH);
   glTranslatef(center[0],center[1],center[2]);
   gluSphere(g.qobj, radius, 15, 10);
   glPopMatrix();
}

void graph_sphere_(double * center, double radius)
{

   gluQuadricCallback(g.qobj, GLU_ERROR, (void (*) ()) errorCallback);
   gluQuadricDrawStyle(g.qobj, GLU_FILL); /* smooth shaded */
   gluQuadricNormals(g.qobj, GLU_SMOOTH);
   glTranslatef(center[0],center[1],center[2]);
   gluSphere(g.qobj, radius, 15, 10);
}


void graph_cylinder( double * begin, double * end, double radius)
{
   int i;
   float r=0;
   glPushMatrix();
   gluQuadricCallback(g.qobj, GLU_ERROR, (void (*) ()) errorCallback);
   gluQuadricDrawStyle(g.qobj, GLU_FILL); /* smooth shaded */
   gluQuadricNormals(g.qobj, GLU_SMOOTH);
   for (i=0; i < 3;i++)
   {
      float diff = (end[i] - begin[i]);
      r += diff * diff;
   }

   r = sqrt(r);


   glTranslatef(begin[0],begin[1],begin[2]);

   {
      float theta = acos((end[2]-begin[2])/r);
      float phi = (180.0/M_PI) * atan2(end[1]-begin[1],end[0]-begin[0]);
      theta *= (180.0/M_PI);
      glRotatef(phi , 0.0, 0.0, 1.0);
      glRotatef(theta , 0.0, 1.0, 0.0);
   }

   glColor3f (0.0, 0.0, 1.0);
   gluCylinder(g.qobj, radius, radius, r, 15, 5);

   glPopMatrix();

}

void reshape1 (int w, int h)
{
   float size=50;
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w <= h)
      glOrtho(-size, size, -size*(GLfloat)h/(GLfloat)w,
         size*(GLfloat)h/(GLfloat)w, -size*4, size * 4);
   else
      glOrtho(-size*(GLfloat)w/(GLfloat)h,
         size*(GLfloat)w/(GLfloat)h, -size, size, -size*4, size * 4);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}
#if 0
void keyboard(unsigned char key, int x, int y)
{
   switch (key) 
   {
      case 27:
	 exit(0);
	 break;
      case 'q':
	 exit(0);
	 break;
      case 'p':
	 sleep(1);
	 break;
      case 'P':
	 sleep(10);
	 break;
      case 's':
	 stop ^= 1;
	 break;
   }

}
#endif
//}
#endif
