#!/usr/bin/python
from scipy import *
from pylab import *
from scipy.integrate import *

graphics = True

omega = 0.9
ampl = 0.01

#This defines a magnetic field that rotates around the z-axis as a function of time t
def mag_field(t):
   Bx = ampl*cos(omega*t)
   By = -ampl*sin(omega*t)
   return array([Bx,By,1.0])


#This is the right hand side of Bloch's equation(s). This is the way that you
#  typically solve first order differential equations. 
#  The magnetization M is a vector with three components, Mx, My, and Mz. But
#  the way this is handled by the algorithm, we identify these three components
#  with elements of an array, so M is described by M[0], M[1], and M[2], that is
#  it's an array with dimension 3.
#  In first order differential equations, the left hand side is the
#  time derivative (of M in this case). You feed this routine to odeint (see below) and it will
#  give you the magnetization M, and a function of time.
def dx_dt(M,t):
   out = zeros((3))
   B = mag_field(t)
   out[0] = gmma * (M[1]*B[2]-M[2]*B[1]) - invT2* M[0]
   out[1] = gmma * (M[2]*B[0]-M[0]*B[2]) - invT2* M[1]
   out[2] = gmma * (M[0]*B[1]-M[1]*B[0]) - invT1* (M[2]-M0)
   return out

M0 = 0
# You can look up the Bloch equations to see how these parameters: gmma, invT1,
# and invT2, are defined and interpreted.

#theta is the starting angle:
theta = pi/4.0
#theta = 0

#m corresponds to an array of magnetizations at different times.
# We start off with it pointing at an angle theta with respect to the z axis.
m = array([[sin(theta),0,cos(theta)]])


gmma = 1.0
invT1 = 0.0
invT2 = 0.0
#How often the graphics is displayed:
display_interval = 2
#time0 is just a counter
time0 = 0
# The time step for the differential equation routine odeint. Smaller numbers are
# more accurate, but this is probably fine:
dt = 0.01



if graphics:
    ion()
    from visual import *
    spin = arrow(pos=(0,0,0), axis=m[-1], shaftwidth=0.1, color=color.red)
#   This sets up the z axis as a reference:
    cyl = cylinder(pos=(0,0,0), axis=(0,0,2.0), radius = 0.05, color=color.blue)
print spin.pos


#iterate over 1000 steps. You can change this to make it longer or shorter:
for t_coarse in arange (0, 1000, dt):
#   odeint wants this array of times:
    if graphics:
       sleep(0.0001)
    t = array([t_coarse,t_coarse+dt])
    #The diff'lequations are integrated. m[-1] means the LAST entry in the 2d
    #array m. That is the last spin configuration that was evovled. This is the
    # starting point for the next iteration: 
    m = odeint(dx_dt, m[-1], t, full_output=False)
    time0 += 1
    if graphics and time0%display_interval == 0:
        spin.axis = m[-1]
       


